import AST
import functions
import globals
import lexer
import Visitor
import utils

class ExpressionEval(Visitor.Visitor):

    def visit_AndExpression(self, astnode, arg):
        true1 = astnode.expr1.visit(self)
        if not true1:
            return False
        return astnode.expr2.visit(self)

    def visit_OrExpression(self, astnode, arg):
        true1 = astnode.expr1.visit(self)
        if true1:
            return True
        return astnode.expr2.visit(self)

    def visit_NotExpression(self, astnode, arg):
        return not astnode.expr.visit(self)

    def visit_FunctionExpression(self, astnode, arg):
        funcname = astnode.funcname
        param_values = [p.visit(self) for p in astnode.params]
        return functions.execute_function(funcname, param_values)

    def visit_CompareExpression(self, astnode, arg):
        value1 = astnode.expr1.visit(self)
        value2 = astnode.expr2.visit(self)
#        print 'Compare', value1, value2
        if astnode.operator_type == lexer.T_LESS_THAN:
            return value1 < value2
        elif astnode.operator_type == lexer.T_LESS_THAN_EQUAL:
            return value1 <= value2
        elif astnode.operator_type == lexer.T_MORE_THAN:
            return value1 > value2
        elif astnode.operator_type == lexer.T_MORE_THAN_EQUAL:
            return value1 >= value2
        elif astnode.operator_type == lexer.T_EQUAL:
            return value1 == value2
        elif astnode.operator_type == lexer.T_NOT_EQUAL:
            return value1 != value2
        else:
            raise ValueError, 'Unknown compare operator (%s)' % astnode.operator_type

    def visit_LiteralExpression(self, astnode, arg):
        if astnode.expr_type == globals.E_INT:
            return int(astnode.strvalue, 10)
        elif astnode.expr_type == globals.E_FLOAT:
            return float(astnode.strvalue)
        elif astnode.expr_type == globals.E_STRING:
            return astnode.strvalue
        elif astnode.expr_type == globals.E_FILENAME:
            return astnode.filename_value
        elif astnode.expr_type == globals.E_DATETIME:
            return astnode.datetime_value
        elif astnode.expr_type == globals.E_STRING_FILENAME_DATETIME:
            return astnode.strvalue
        else:
            raise TypeError, 'Cannot evaluate atom "%s"' % astnode.strvalue

    def visit_VariableExpression(self, astnode, arg):
        return astnode.variable_obj.value