import datetime
import os
import time

import globals

def f_age(filename):
    modtime = os.path.getmtime(filename)
    age_seconds = time.time() - modtime
    return age_seconds / 86400.0

def f_base(filename):
    return ''

def f_date(filename):
    modtime = os.path.getmtime(filename)
    dt = datetime.datetime.fromtimestamp(modtime)
    return dt #.isoformat()

def f_exists(filename):
    return os.path.exists(filename)

def f_extract(timestamp, part):
    if not isinstance(timestamp, datetime.datetime):
        raise TypeError, 'timestamp must be datetime.datetime instance'
    part = part.lower()

    if part == 'year':
        return timestamp.year
    elif part == 'month':
        return timestamp.month
    elif part == 'day':
        return timestamp.day
    elif part in ('hour', 'hours'):
        return timestamp.hour
    elif part in ('minute', 'minutes'):
        return timestamp.minute
    elif part in ('second', 'seconds'):
        return timestamp.second
    elif part == 'weekday':
        return timestamp.isoweekday()
    elif part == 'week':
        isoyear, week, weekday = timestamp.isocalendar()
        return week
    else:
        raise ValueError, 'Invalid part specifier: "%s"' % part

def f_now():
    return datetime.datetime.now() #.isoformat()

def f_size(filename):
    return os.path.getsize(filename)

# 'name': ((params), output)
_FUNCTION_TYPES = {
    'age':      (f_age,     [globals.E_FILENAME], globals.E_FLOAT),
    'base':     (f_base,    [globals.E_FILENAME], globals.E_FILENAME),
    'date':     (f_date,    [globals.E_FILENAME], globals.E_DATETIME),
    'exists':   (f_exists,  [globals.E_FILENAME], globals.E_BOOLEAN),
    'extract':  (f_extract, [globals.E_DATETIME,  globals.E_STRING], globals.E_INT),
    'now':      (f_now,     [],                   globals.E_DATETIME),
    'size':     (f_size,    [globals.E_FILENAME], globals.E_INT),
}

def get_function_return_type(funcname):
    if funcname.lower() not in _FUNCTION_TYPES:
        raise ValueError, 'Unknown function: %s' % funcname
    return _FUNCTION_TYPES[funcname.lower()][2]

def get_function_param_types(funcname):
    if funcname.lower() not in _FUNCTION_TYPES:
        raise ValueError, 'Unknown function: %s' % funcname
    return _FUNCTION_TYPES[funcname.lower()][1]

def execute_function(funcname, params):
    if funcname.lower() not in _FUNCTION_TYPES:
        raise ValueError, 'Unknown function: %s' % funcname

    if None in params:
        raise ValueError, 'Parameter is None: %s' % params

    func, paramtypes, returntype = _FUNCTION_TYPES[funcname.lower()]
#    print 'Execute %s(%s)' % (funcname, params)
    if paramtypes == [globals.E_FILENAME] and '/' not in params[0]:
        print 'Warning: calling filename function (%s) with bare string literal' % funcname
    return func(*params)
