import datetime
import os
import re

def combine_rootdir(rootdir, path):
    if path.endswith('/') and len(path) > 1:
        path = path[:-1]

    if filename_is_absolute(path):
        return path.replace('\\', '/')

    if path.startswith('/'):
        path = path[1:]
    if not rootdir.endswith('/'):
        rootdir += '/'
    return (rootdir + path).replace('\\', '/')

def pathjoin(path1, path2):
    path1 = path1.replace('\\', '/')
    path2 = path2.replace('\\', '/')
    return os.path.join(path1, path2).replace('\\', '/')

def abspath(path):
    normpath = path.replace('\\', '/')
    return os.path.abspath(normpath).replace('\\', '/')

def calc_fullglob_innerdir(rootdir, globpattern):
    """Calculate absolute (full) glob pattern from relative pattern."""

    rootdir = abspath(rootdir)

    if not rootdir.endswith('/'):
        rootdir = rootdir + '/'
    if globpattern.endswith('/'):
        globpattern = globpattern[:-1]

    innerdir = abspath(calc_innerdir(rootdir + globpattern))

    if globpattern.startswith('*') and len(rootdir)!=1 and (len(rootdir)!=3 or rootdir[1]!=':'):
        # To properly support patterns beginning with "*", we
        # produce the fullglob "/rootdir*" instead of "/rootdir/*";
        # but not if the rootdir is root directory of file system.
        # (On Windows "d:/", on Unix "/".)
        rootdir = rootdir[:-1] # remove tailing "/"

    fullglob = abspath(rootdir + globpattern)
    return fullglob, innerdir

def calc_innerdir(fullglob):
    fullglob = fullglob.replace('\\', '/')
    inner_end_index = 0
    for i, char in enumerate(fullglob):
        if char in '*?[':
            break
        elif char == '/':
            inner_end_index = i
    else:
        inner_end_index = len(fullglob)-1

    innerdir = fullglob[:inner_end_index+1]
    if innerdir.endswith('/') and len(innerdir)>1 and not innerdir[-2] == ':':
        innerdir = innerdir[:-1]
    return innerdir

def filename_is_absolute(pattern):
    norm_pattern = pattern.replace('\\', '/')
    if len(norm_pattern) >= 3 and norm_pattern[0].isalpha() and norm_pattern[1] == ':' and norm_pattern[2] == '/':
        # Windows file name
        return True
    return norm_pattern.startswith('/')


def format_size(size):
    if size < 1000:
        return '%d B' % size
    elif size < 1000000:
        return '%.1f KB' % (size/1024.0)
    elif size < 1000000000:
        return '%.1f MB' % (size/2.0**20)
    else:
        return '%.1f GB' % (size/2.0**30)

_DATETIME_PATTERNS = [
    '(?P<year>\d{4})$', # yyyy
    '(?P<year>\d{4})-(?P<month>\d{1,2})$', # yyyy-mm
    '(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})$', # yyyy-mm-dd
    '(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})[ ](?P<hour>\d{1,2}):(?P<minute>\d{1,2})$', # yyyy-mm-dd hh:mm
    '(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})[ ](?P<hour>\d{1,2}):(?P<minute>\d{1,2}):(?P<second>\d{1,2})$', # yyyy-mm-dd hh:mm:ss
]

def parse_datetime(string):
    """Parse datetime string, return `datetime.datetime` instance.
    Raise ValueError if the string can't be parsed."""
    for pattern in _DATETIME_PATTERNS:
        match = re.match(pattern, string)
        if match:
            groupdict = match.groupdict()
            year = int(groupdict.get('year'))
            month = int(groupdict.get('month', 1))
            day = int(groupdict.get('day', 1))
            hour = int(groupdict.get('hour', 0))
            minute = int(groupdict.get('minute', 0))
            second = int(groupdict.get('second', 0))
            return datetime.datetime(year, month, day, hour, minute, second)
    raise ValueError, 'String "%s" cannot be parsed as datetime' % string
